import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IMetadataProperty } from '../metadata-property.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../metadata-property.test-samples';

import { MetadataPropertyService } from './metadata-property.service';

const requireRestSample: IMetadataProperty = {
  ...sampleWithRequiredData,
};

describe('MetadataProperty Service', () => {
  let service: MetadataPropertyService;
  let httpMock: HttpTestingController;
  let expectedResult: IMetadataProperty | IMetadataProperty[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(MetadataPropertyService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a MetadataProperty', () => {
      const metadataProperty = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(metadataProperty).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a MetadataProperty', () => {
      const metadataProperty = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(metadataProperty).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a MetadataProperty', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of MetadataProperty', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a MetadataProperty', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addMetadataPropertyToCollectionIfMissing', () => {
      it('should add a MetadataProperty to an empty array', () => {
        const metadataProperty: IMetadataProperty = sampleWithRequiredData;
        expectedResult = service.addMetadataPropertyToCollectionIfMissing([], metadataProperty);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(metadataProperty);
      });

      it('should not add a MetadataProperty to an array that contains it', () => {
        const metadataProperty: IMetadataProperty = sampleWithRequiredData;
        const metadataPropertyCollection: IMetadataProperty[] = [
          {
            ...metadataProperty,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addMetadataPropertyToCollectionIfMissing(metadataPropertyCollection, metadataProperty);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a MetadataProperty to an array that doesn't contain it", () => {
        const metadataProperty: IMetadataProperty = sampleWithRequiredData;
        const metadataPropertyCollection: IMetadataProperty[] = [sampleWithPartialData];
        expectedResult = service.addMetadataPropertyToCollectionIfMissing(metadataPropertyCollection, metadataProperty);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(metadataProperty);
      });

      it('should add only unique MetadataProperty to an array', () => {
        const metadataPropertyArray: IMetadataProperty[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const metadataPropertyCollection: IMetadataProperty[] = [sampleWithRequiredData];
        expectedResult = service.addMetadataPropertyToCollectionIfMissing(metadataPropertyCollection, ...metadataPropertyArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const metadataProperty: IMetadataProperty = sampleWithRequiredData;
        const metadataProperty2: IMetadataProperty = sampleWithPartialData;
        expectedResult = service.addMetadataPropertyToCollectionIfMissing([], metadataProperty, metadataProperty2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(metadataProperty);
        expect(expectedResult).toContain(metadataProperty2);
      });

      it('should accept null and undefined values', () => {
        const metadataProperty: IMetadataProperty = sampleWithRequiredData;
        expectedResult = service.addMetadataPropertyToCollectionIfMissing([], null, metadataProperty, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(metadataProperty);
      });

      it('should return initial array if no MetadataProperty is added', () => {
        const metadataPropertyCollection: IMetadataProperty[] = [sampleWithRequiredData];
        expectedResult = service.addMetadataPropertyToCollectionIfMissing(metadataPropertyCollection, undefined, null);
        expect(expectedResult).toEqual(metadataPropertyCollection);
      });
    });

    describe('compareMetadataProperty', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareMetadataProperty(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareMetadataProperty(entity1, entity2);
        const compareResult2 = service.compareMetadataProperty(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareMetadataProperty(entity1, entity2);
        const compareResult2 = service.compareMetadataProperty(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareMetadataProperty(entity1, entity2);
        const compareResult2 = service.compareMetadataProperty(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
