import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IOrganizationType } from '../organization-type.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../organization-type.test-samples';

import { OrganizationTypeService } from './organization-type.service';

const requireRestSample: IOrganizationType = {
  ...sampleWithRequiredData,
};

describe('OrganizationType Service', () => {
  let service: OrganizationTypeService;
  let httpMock: HttpTestingController;
  let expectedResult: IOrganizationType | IOrganizationType[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(OrganizationTypeService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a OrganizationType', () => {
      const organizationType = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(organizationType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a OrganizationType', () => {
      const organizationType = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(organizationType).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a OrganizationType', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of OrganizationType', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a OrganizationType', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addOrganizationTypeToCollectionIfMissing', () => {
      it('should add a OrganizationType to an empty array', () => {
        const organizationType: IOrganizationType = sampleWithRequiredData;
        expectedResult = service.addOrganizationTypeToCollectionIfMissing([], organizationType);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(organizationType);
      });

      it('should not add a OrganizationType to an array that contains it', () => {
        const organizationType: IOrganizationType = sampleWithRequiredData;
        const organizationTypeCollection: IOrganizationType[] = [
          {
            ...organizationType,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addOrganizationTypeToCollectionIfMissing(organizationTypeCollection, organizationType);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a OrganizationType to an array that doesn't contain it", () => {
        const organizationType: IOrganizationType = sampleWithRequiredData;
        const organizationTypeCollection: IOrganizationType[] = [sampleWithPartialData];
        expectedResult = service.addOrganizationTypeToCollectionIfMissing(organizationTypeCollection, organizationType);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(organizationType);
      });

      it('should add only unique OrganizationType to an array', () => {
        const organizationTypeArray: IOrganizationType[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const organizationTypeCollection: IOrganizationType[] = [sampleWithRequiredData];
        expectedResult = service.addOrganizationTypeToCollectionIfMissing(organizationTypeCollection, ...organizationTypeArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const organizationType: IOrganizationType = sampleWithRequiredData;
        const organizationType2: IOrganizationType = sampleWithPartialData;
        expectedResult = service.addOrganizationTypeToCollectionIfMissing([], organizationType, organizationType2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(organizationType);
        expect(expectedResult).toContain(organizationType2);
      });

      it('should accept null and undefined values', () => {
        const organizationType: IOrganizationType = sampleWithRequiredData;
        expectedResult = service.addOrganizationTypeToCollectionIfMissing([], null, organizationType, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(organizationType);
      });

      it('should return initial array if no OrganizationType is added', () => {
        const organizationTypeCollection: IOrganizationType[] = [sampleWithRequiredData];
        expectedResult = service.addOrganizationTypeToCollectionIfMissing(organizationTypeCollection, undefined, null);
        expect(expectedResult).toEqual(organizationTypeCollection);
      });
    });

    describe('compareOrganizationType', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareOrganizationType(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareOrganizationType(entity1, entity2);
        const compareResult2 = service.compareOrganizationType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareOrganizationType(entity1, entity2);
        const compareResult2 = service.compareOrganizationType(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareOrganizationType(entity1, entity2);
        const compareResult2 = service.compareOrganizationType(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
